{***************************************************************
 *
 * Project name      : OpenALConsole
 * Author            : Amresh Ramachandran amreshr@hotmail.com
 * Project directory : D:\Program Files\Borland\Delphi5\Projects\OpenALConsole
 * Created           : 16-Oct-2001
 * History           :
 * Saturday, October 20, 2001 16:37
 * Last modification
 ****************************************************************}
program OpenALConsole;
{$APPTYPE CONSOLE}

uses Windows, SysUtils, Classes, AlTypes, Al, AlcTypes, Alc, Alut, EAX;

const
  NUM_BUFFERS = 7; // Number of buffers to be Generated
  BSIZE = 20000;
  NUMBUFFERS = 4;

  listenerPos: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
  listenerVel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
  listenerOri: array[0..5] of TALfloat = (0.0, 0.0, -1.0, 0.0, 1.0, 0.0);

type
  TWaveStruct = record
    riff: array[0..3] of TALubyte; // 'RIFF'
    riffSize: TALsizei;
    wave: array[0..3] of TALubyte; // 'WAVE'
    fmt: array[0..3] of TALubyte; // 'fmt '
    fmtSize: TALuint;
    Format: TALushort;
    Channels: TALushort;
    SamplesPerSec: TALuint;
    BytesPerSec: TALuint;
    BlockAlign: TALushort;
    BitsPerSample: TALushort;
    data: array[0..3] of TALubyte; // 'data'
    dataSize: TALuint;
  end;

  TKeyShiftState = (kssRightAlt, kssLeftAlt,
    kssRightCtrl, kssLeftCtrl, kssShift, kssNumLock,
    kssScrollLock, kssCapsLock, kssEnhancedKey);

  TKeyShiftSet = set of TKeyShiftState;

  PKeyEventRecordFix = ^TKeyEventRecordFix;
  TKeyEventRecordFix = packed record
    bKeyDown: BOOL;
    wRepeatCount: Word;
    wVirtualKeyCode: Word;
    wVirtualScanCode: Word;
    case Integer of
      0: (
        UnicodeChar: WCHAR;
        dwControlKeyState: DWORD);
      1: (
        AsciiChar: CHAR)
  end;
var
  ch: Char;
  VirtualKeyCode: Word;
  VirtualScanCode: Word;

//  szFnName: PALubyte;
  g_bEAX: TALBoolean;
  size, freq: TALsizei;
  format: TALenum;
  loop: TALint;
  Context: TALCcontext;
  Device: TALCdevice;
  error: TALint;
  g_Buffers: array[0..NUM_BUFFERS - 1] of TALuint; // Array of Buffer IDs
  dataBuffer: TALvoid;

//function KeyPressed: boolean; assembler;
//asm
//	mov AH,$01;
//	int $16
//	mov AL, TRUE;
//	JNZ @_End;
//	mov AL, FALSE;
//	@_End:
//end;

function VetoOnVKCode(const VKCode: Word): Boolean;
const
  NumVetoKeys = 6;
  VetoKey: array[1..NumVetoKeys] of Word = (VK_CONTROL, VK_SHIFT,
    VK_MENU, VK_CAPITAL,
    VK_NUMLOCK, VK_SCROLL);
var
  iKey: Integer;
begin
  for iKey := 1 to NumVetoKeys do
    if VKCode = VetoKey[iKey] then
    begin
      Result := True;
      Exit
    end;
  Result := False
end;

function ReadKey: Char;
var
  NumRead: Cardinal;
  HConsoleInput: THandle;
  InputRec: TInputRecord;
  KeyShiftState: TKeyShiftSet;
begin
  HConsoleInput := GetStdHandle(STD_INPUT_HANDLE);
  repeat
    if WaitForSingleObject(HConsoleInput, INFINITE) <>
      WAIT_OBJECT_0 then
      raise Exception.Create('Invalid handle for console input');
    if ReadConsoleInput(HConsoleInput, InputRec, 1, NumRead) and
      (InputRec.EventType = KEY_EVENT) then
    begin
      with PKeyEventRecordFix(@InputRec.Event)^ do
        if bKeyDown and not VetoOnVKCode(wVirtualKeyCode) then
        begin
          VirtualScanCode := wVirtualScanCode;
          VirtualKeyCode := wVirtualKeyCode;
          KeyShiftState := TKeyShiftSet(LongRec(dwControlKeyState).Lo);
          Result := AsciiChar;
          Exit;
        end;
    end;
  until False
end;

function InitAL: Boolean;
begin
 //Open Device
  Device := alcOpenDevice('DirectSound3D');
  if Device = nil then
  begin
    Writeln('Unable to intialize OpenAL');
    Result := False;
    Exit;
  end;
 //Create Context
  Context := alcCreateContext(Device, nil);
  if Context = nil then
  begin
    Writeln('Unable to create OpenAL context');
    Result := False;
    Exit;
  end;
 //Make context current
  alcMakeContextCurrent(Context);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('Unable to make context current');
    Result := False;
    Exit;
  end;
  Result := True;
end;

procedure Cleanup;
begin
  Writeln;
  Writeln('Cleaning up...');
  Context := alcGetCurrentContext();
 //Get active context
  Writeln('...Getting current context');
 //Get device for active context
  Writeln('...Getting device of context');
  Device := alcGetContextsDevice(Context);
 //Release context(s)
  Writeln('...Destroying contexts');
  alcDestroyContext(Context);
 //Close device
  Writeln('...Closing Device,  please wait...');
  alcCloseDevice(Device);
end;

function SetListernerAttributes: Boolean;
begin
  Writeln;
 //Position
  alListenerfv(AL_POSITION, @listenerPos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alListenerfv POSITION error');
    Result := False;
    Exit;
  end;
 //Velocity
  alListenerfv(AL_VELOCITY, @listenerVel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alListenerfv VELOCITY error');
    Result := False;
    Exit;
  end;
 //Orientation
  alListenerfv(AL_ORIENTATION, @listenerOri);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alListenerfv ORIENTATION error');
    Result := False;
    Exit;
  end;
  Result := True;
end;

function LoadSample(sampleName: string; bufferNumber: Integer): Boolean;
begin
  Writeln;
  Writeln('Loading ' + sampleName + ' to buffer: ' + InttoStr(bufferNumber));
 // Load sampleName.wav
  alutLoadWAVFile(sampleName, format, dataBuffer, size, freq, loop);
  error := alGetError;
  if (error <> AL_NO_ERROR) then
  begin
    Writeln('alutLoadWAVFile ' + sampleName + ' Error: ', error);
 // Delete Buffers
    alDeleteBuffers(NUM_BUFFERS, @g_Buffers);
    Result := False;
    Exit;
  end;

 // Copy sampleName.wav data into AL Buffer <bufferNumber>
  alBufferData(g_Buffers[bufferNumber], format, dataBuffer, size, freq);
  error := alGetError;
  if (error <> AL_NO_ERROR) then
  begin
    Writeln('alBufferData buffer ' + IntToStr(bufferNumber) + ' Error: ',
      error);
 // Delete buffers
    alDeleteBuffers(NUM_BUFFERS, @g_Buffers);
    Result := False;
    Exit;
  end;

 // Unload sampleName.wav
  alutUnloadWAV(format, dataBuffer, size, freq);
  error := alGetError;
  if (error <> AL_NO_ERROR) then
  begin
    Writeln('alutUnloadWAV: Error: ', error);
 // Delete buffers
    alDeleteBuffers(NUM_BUFFERS, @g_Buffers);
    Result := False;
    Exit;
  end;
  Result := True;
end;

(*
 Position Test
 Creates 2 Sources - one to the front right of the listener, and one to the rear left of the listener
*)

procedure PositionTest;
const
	source0Pos: array[0..2] of TALfloat = (-2.0, 0.0, 2.0);
	// Behind and to the left of the listener
	source0Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);

	source1Pos: array[0..2] of TALfloat = (2.0, 0.0, -2.0);
	// Front and to the right of the listener
	source1Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
var
	key: Char;
	source: array[0..1] of TALuint;
begin
	Writeln;
	alGenSources(2, @source);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alGenSources Error: ' + IntToStr(error));
		exit;
	end;

	alSourcef(source[0], AL_PITCH, 1.0);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcef 0 AL_PITCH Error: ' + IntToStr(error));
		exit;
	end;
	alSourcef(source[0], AL_GAIN, 1.0);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcef 0 AL_GAIN Error: ' + IntToStr(error));
		exit;
	end;
	alSourcefv(source[0], AL_POSITION, @source0Pos);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcefv 0 AL_POSITION Error: ' + IntToStr(error));
		exit;
	end;
	alSourcefv(source[0], AL_VELOCITY, @source0Vel);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcefv 0 AL_VELOCITY Error: ' + IntToStr(error));
		exit;
	end;
	alSourcei(source[0], AL_BUFFER, g_Buffers[1]);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcei 0 AL_BUFFER Error: ' + IntToStr(error));
		exit;
	end;
	alSourcei(source[0], AL_LOOPING, AL_TRUE);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcei 0 AL_LOOPING Error: ' + IntToStr(error));
		exit;
  end;

  alSourcef(source[1], AL_PITCH, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 1 AL_PITCH Error: ' + IntToStr(error));
    exit;
  end;
  alSourcef(source[1], AL_GAIN, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 1 AL_GAIN Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[1], AL_POSITION, @source0Pos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 1 AL_POSITION Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[1], AL_VELOCITY, @source0Vel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 1 AL_VELOCITY Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[1], AL_BUFFER, g_Buffers[1]);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 1 AL_BUFFER Error: ' + IntToStr(error));
    exit;
  end;
	alSourcei(source[1], AL_LOOPING, AL_TRUE);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 1 AL_LOOPING Error: ' + IntToStr(error));
    exit;
  end;

  Writeln('Position Test');
  Writeln('Press "1" to play source 0 (looping) rear left of listener');
  Writeln('Press "2" to play source 1 (looping) front right of listener');
	Writeln('Press "3" to stop source 0');
	Writeln('Press "4" to stop source 1');
	Writeln('Press "q" to quit');
	Writeln;

	repeat
		Key := ReadKey;
    Key := UpCase(Key);
    case (Key) of
      '1': begin
          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 0 : ' + InttoStr(error));
        end;
      '2': begin
          alSourcePlay(source[1]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 1 : ' + InttoStr(error));
        end;
      '3': begin
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop source 0 : ' + InttoStr(error));
        end;
      '4': begin
          alSourceStop(source[1]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop source 1 : ' + InttoStr(error));
        end;
    end;
  until (Key = 'Q');

  alSourceStopv(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alSourceStopv 2 : ' + IntToStr(error));

  alDeleteSources(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alDeleteSources 2 : ' + IntToStr(error));
end;

(*
 Looping Test
 Tests the ability to switch Looping ON and OFF for a particular source, either before or during
 Playback.  (If looping is switched off during playback, the buffer should finish playing until
 the end of the sample.)
*)

procedure LoopingTest;
const
  source0Pos: array[0..2] of TALfloat = (-2.0, 0.0, -2.0);
    // Behind and to the left of the listener
  source0Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
  source1Pos: array[0..3] of TALfloat = (2.0, 0, 0, -2.0);
    // Front and to the right of the listener
  source1Vel: array[0..3] of TALfloat = (0.0, 0.0, 0, 0);
var
  key: Char;
  source: array[0..1] of TALuint;
  bLooping0,
    bLooping1: Integer;
begin
  bLooping0 := AL_FALSE;
  bLooping1 := AL_FALSE;
  Writeln;
 //Clear Error code;
  alGetError;

  alGenSources(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alGenSources Error: ' + IntToStr(error));
    exit;
  end;

  alSourcef(source[0], AL_PITCH, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_PITCH Error: ' + IntToStr(error));
    exit;
  end;
  alSourcef(source[0], AL_GAIN, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_GAIN Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_POSITION, @source0Pos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_POSITION Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_VELOCITY, @source0Vel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_VELOCITY Error: ' + IntToStr(error));
    exit;
  end;
	alSourcei(source[0], AL_BUFFER, g_Buffers[0]);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcei 0 AL_BUFFER buffer 0 Error: ' + IntToStr(error));
		exit;
	end;
	alSourcei(source[0], AL_LOOPING, AL_FALSE);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcei 0 AL_LOOPING Error: ' + IntToStr(error));
		exit;
	end;

	alSourcef(source[1], AL_PITCH, 1.0);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcef 1 AL_PITCH Error: ' + IntToStr(error));
		exit;
	end;
	alSourcef(source[1], AL_GAIN, 1.0);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcef 1 AL_GAIN Error: ' + IntToStr(error));
		exit;
	end;
	alSourcefv(source[1], AL_POSITION, @source0Pos);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcefv 1 AL_POSITION Error: ' + IntToStr(error));
		exit;
	end;
	alSourcefv(source[1], AL_VELOCITY, @source0Vel);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcefv 1 AL_VELOCITY Error: ' + IntToStr(error));
		exit;
	end;
	alSourcei(source[1], AL_BUFFER, g_Buffers[1]);
	error := alGetError;
	if error <> AL_NO_ERROR then
	begin
		Writeln('alSourcei 1 AL_BUFFER buffer 1 Error: ' + IntToStr(error));
		exit;
	end;
	alSourcei(source[1], AL_LOOPING, AL_FALSE);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 1 AL_LOOPING Error: ' + IntToStr(error));
    exit;
  end;

  Writeln('Looping Test');
  Writeln('Press "1" to play source 0 once (single shot)');
  Writeln('Press "2" to toggle looping on source 0');
  Writeln('Press "3" to play source 1 once (single shot)');
  Writeln('Press "4" to toggle looping on source 1');
  Writeln('Press "q" to quit');
  Write('Source 0 : Not looping Source 1 : Not looping' + #13);

  repeat
    Key := ReadKey;
    Key := UpCase(Key);
    case (Key) of
      '1': begin
          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 0 : ' + InttoStr(error));
        end;
      '2': begin
          if (bLooping0 = AL_FALSE) then
          begin
            bLooping0 := AL_TRUE;
            if (bLooping1 <> 0) then
              Write('Source 0 : Looping     Source 1 : Looping    ' + #13)
            else
              Write('Source 0 : Looping     Source 1 : Not looping' + #13);
          end
          else
          begin
            bLooping0 := AL_FALSE;
            if (bLooping1 <> 0) then
              Write('Source 0 : Not looping Source 1 : Looping    ' + #13)
            else
              Write('Source 0 : Not looping Source 1 : Not looping' + #13);
          end;
          alSourcei(source[0], AL_LOOPING, bLooping0);
        end;
      '3': begin
          alSourcePlay(source[1]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 1 : ' + InttoStr(error));
        end;
      '4': begin
          if (bLooping1 = AL_FALSE) then
          begin
            bLooping1 := AL_TRUE;
            if (bLooping0 <> 0) then
              Write('Source 0 : Looping     Source 1 : Looping    ' + #13)
            else
              Write('Source 0 : Not looping Source 1 : Looping    ' + #13);
          end
          else
          begin
            bLooping1 := AL_FALSE;
            if (bLooping0 <> 0) then
              Write('Source 0 : Looping     Source 1 : Not looping' + #13)
            else
              Write('Source 0 : Not looping Source 1 : Not looping' + #13);
          end;
          alSourcei(source[0], AL_LOOPING, bLooping0);
        end;
    end;
  until (Key = 'Q');

  alSourceStopv(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alSourceStopv 2 : ' + IntToStr(error));

  alDeleteSources(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alDeleteSources 2 : ' + IntToStr(error));
end;

(*
 EAXTest (Only runs if the EAX Extension is present)
 Uses 2 Sources to test out EAX 2.0 Reverb, Occlusion and Obstruction.  Also tests the use of the
 DEFERRED flag in EAX.
*)

procedure EAXTest;
const
  source0Pos: array[0..2] of TALfloat = (-2.0, 0.0, 2.0);
  // Behind and to the left of the listener
  source0Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);

  source1Pos: array[0..2] of TALfloat = (2.0, 0.0, -2.0);
    // Front and right of the listener
  source1Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
var
  error: TALint;
  source: array[0..2] of TALuint;
  Env: TALuint;
  Room: TALint;
  Occlusion: TALint;
  Obstruction: TALint;
  eaxBufferProp0: TEaxBufferProperties;
  Key: Char;
begin
  alGenSources(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alGenSources Error: ' + IntToStr(error));
    exit;
  end;

  alSourcef(source[0], AL_PITCH, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_PITCH Error: ' + IntToStr(error));
    exit;
  end;
  alSourcef(source[0], AL_GAIN, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_GAIN Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_POSITION, @source0Pos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_POSITION Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_VELOCITY, @source0Vel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_VELOCITY Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[0], AL_BUFFER, g_Buffers[0]);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 0 AL_BUFFER Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[0], AL_LOOPING, AL_TRUE);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 0 AL_LOOPING Error: ' + IntToStr(error));
    exit;
  end;

  alSourcef(source[1], AL_PITCH, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 1 AL_PITCH Error: ' + IntToStr(error));
    exit;
  end;
  alSourcef(source[1], AL_GAIN, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 1 AL_GAIN Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[1], AL_POSITION, @source0Pos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 1 AL_POSITION Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[1], AL_VELOCITY, @source0Vel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 1 AL_VELOCITY Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[1], AL_BUFFER, g_Buffers[1]);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 1 AL_BUFFER Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[1], AL_LOOPING, AL_TRUE);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 1 AL_LOOPING Error: ' + IntToStr(error));
    exit;
  end;

  Writeln;
  Writeln('EAX Test');
  Writeln('Press "1" to play source 0 (looping)');
  Writeln('Press "2" to play source 1 once (single shot)');
  Writeln('Press "3" to stop source 0');
  Writeln('Press "4" to stop source 1');
  Writeln('Press "5" to add Hangar reverb (DEFERRED)');
  Writeln('Press "6" to remove reverb (DEFERRED)');
  Writeln('Press "7" to occlude source 0 (DEFERRED)');
  Writeln('Press "8" to remove occlusion from source 0 (DEFERRED)');
  Writeln('Press "9" to obstruct source 1 (IMMEDIATE)');
  Writeln('Press "0" to remove obstruction from source 1 (IMMEDIATE)');
  Writeln('Press "c" to COMMIT EAX settings');
  Writeln('Press "q" to quit');
  Writeln;

  repeat
    Key := ReadKey;
    Key := UpCase(Key);
    case Key of
      '1': begin
          alSourcePlay(source[0]);
        end;
      '2': begin
          alSourcePlay(source[1]);
        end;
      '3': begin
          alSourceStop(source[0]);
        end;
      '4': begin
          alSourceStop(source[1]);
        end;
      '5': begin
          Env := EAX_ENVIRONMENT_HANGAR;
          eaxSet(DSPROPSETID_EAX20_ListenerProperties,
            DSPROPERTY_EAXLISTENER_ENVIRONMENT or
            DSPROPERTY_EAXLISTENER_DEFERRED,
            0, @Env, sizeof(TALuint));
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('eaxSet EAXLISTENER_ENVIRONMENT or EAXLISTENER_DEFERRED : '
              +
              IntToStr(error));
        end;
      '6': begin
          Room := -10000;
          eaxSet(DSPROPSETID_EAX20_ListenerProperties,
            DSPROPERTY_EAXLISTENER_ROOM or DSPROPERTY_EAXLISTENER_DEFERRED, 0,
            @Room, sizeof(TALint));
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('eaxSet EAXLISTENER_ROOM or EAXLISTENER_DEFERRED : ' +
              IntToStr(error));
        end;
      '7': begin
          eaxGet(DSPROPSETID_EAX20_BufferProperties,
            DSPROPERTY_EAXBUFFER_ALLPARAMETERS, source[0],
            @eaxBufferProp0, sizeof(TEAXBufferProperties));
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('eaxGet EAXBUFFER_ALLPARAMETERS : ' + IntToStr(error));
          eaxBufferProp0.lOcclusion := -5000;
          eaxSet(DSPROPSETID_EAX20_BufferProperties,
            DSPROPERTY_EAXBUFFER_ALLPARAMETERS or DSPROPERTY_EAXBUFFER_DEFERRED,
            source[0], @eaxBufferProp0, sizeof(TEAXBUFFERPROPERTIES));
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('eaxSet EAXBUFFER_ALLPARAMETERS or EAXBUFFER_DEFERRED : ' +
              IntToStr(error));
        end;
      '8': begin
          Occlusion := 0;
          eaxSet(DSPROPSETID_EAX20_BufferProperties,
            DSPROPERTY_EAXBUFFER_OCCLUSION or DSPROPERTY_EAXBUFFER_DEFERRED,
            source[0], @Occlusion, sizeof(TALint));
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('eaxSet EAXBUFFER_OCCLUSION or EAXBUFFER_DEFERRED : ' +
              IntToStr(error));
        end;

      '9': begin
          Obstruction := -5000;
          eaxSet(DSPROPSETID_EAX20_BufferProperties,
            DSPROPERTY_EAXBUFFER_OBSTRUCTION, source[1],
            @Obstruction, sizeof(TALint));
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('eaxSet EAXBUFFER_OBSTRUCTION: ' + IntToStr(error));
        end;

      '0': begin
          Obstruction := 0;
          eaxSet(DSPROPSETID_EAX20_BufferProperties,
            DSPROPERTY_EAXBUFFER_OBSTRUCTION, source[1],
            @Obstruction, sizeof(TALint));
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('eaxSet EAXBUFFER_OBSTRUCTION: ' + IntToStr(error));
        end;

      'C': begin
 // Commit settings on source 0
          eaxSet(DSPROPSETID_EAX20_BufferProperties,
            DSPROPERTY_EAXBUFFER_COMMITDEFERREDSETTINGS,
            source[0], nil, 0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('eaxSet EAXBUFFER_COMMITDEFERREDSETTINGS: ' +
              IntToStr(error));

 // Commit Listener settings
          eaxSet(DSPROPSETID_EAX20_ListenerProperties,
            DSPROPERTY_EAXLISTENER_COMMITDEFERREDSETTINGS, 0, nil, 0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('eaxSet EAXLISTENER_COMMITDEFERREDSETTINGSENVIRONMENT: ' +
              IntToStr(error));
        end;
    end;
  until (Key = 'Q');
  alSourceStopv(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alSourceStopv 2 : ' + IntToStr(error));

  alDeleteSources(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alDeleteSources 2 : ' + IntToStr(error));
end;

(*
 QueueTest
 Tests the ability to queue and unqueue a sequence of buffers on a Source. (Buffers can only be
 unqueued if they have been PROCESSED by a Source.)
*)

procedure QueueTest;
const
  source0Pos: array[0..2] of TALfloat = (0.0, 0.0, -2.0);
 // Immediately in front of listener
  source0Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
var
  source: array[0..0] of TALuint;
  Key: TALubyte;
  buffers: array[0..4] of TALuint;
  buffersremoved: array of TALuint;
  bLooping: TALint;
  BuffersInQueue, BuffersProcessed: TALint;
  Buffer: TALuint;
  i: TALint;
begin
  alGetError;

  alGenSources(1, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alGenSources Error: ' + IntToStr(error));
    exit;
  end;

  alSourcef(source[0], AL_PITCH, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_PITCH Error: ' + IntToStr(error));
    exit;
  end;
  alSourcef(source[0], AL_GAIN, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_GAIN Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_POSITION, @source0Pos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_POSITION Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_VELOCITY, @source0Vel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_VELOCITY Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[0], AL_LOOPING, AL_FALSE);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 0 AL_LOOPING Error: ' + IntToStr(error));
    exit;
  end;

  bLooping := AL_FALSE;

  Writeln;
  Writeln('Queue Test');
  Writeln('Press "1" to start playing source 0');
  Writeln('Press "2" to stop source 0');
  Writeln('Press "3" to toggle looping on source 0');
  Writeln('Press "4" to queue 4 buffers on source 0');
  Writeln('Press "5" to queue 1st buffer on source 0');
  Writeln('Press "6" to queue 2nd buffer on source 0');
  Writeln('Press "7" to queue 3rd buffer on source 0');
  Writeln('Press "8" to queue 4th buffer on source 0');
  Writeln('Press "9" to queue 5th buffer (Buffer 0) on source 0');
  Writeln('Press "0" to display stats');

  Writeln('Press "a" to unqueue first Buffer');
  Writeln('Press "b" to unqueue first 2 Buffers');
  Writeln('Press "c" to unqueue first 3 Buffers');
  Writeln('Press "d" to unqueue first 4 Buffers');
  Writeln('Press "e" to unqueue first 5 Buffers');
  Writeln('Press "f" to unqueue all buffers');

  Writeln('Press "q" to quit');

  Write('Source 0 not looping' + #13);

  buffers[0] := g_Buffers[2];
  buffers[1] := g_Buffers[3];
  buffers[2] := g_Buffers[4];
  buffers[3] := g_Buffers[5];
	buffers[4] := 0;

  repeat
    Key := ReadKey;
    Key := UpCase(Key);
    case Key of
      '1': begin
          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 0: ', error);
        end;
      '2': begin
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop source 0: ', error);
        end;
      '3': begin
          if (bLooping = AL_TRUE) then
          begin
            bLooping := AL_FALSE;
            Write('Source 0 not looping' + #13);
          end
          else
          begin
            bLooping := AL_TRUE;
            Write('Source 0 looping    ' + #13);
          end;
          alSourcei(source[0], AL_LOOPING, bLooping);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei AL_LOOPING: ', error);
        end;
      '4': begin
          alSourceQueueBuffers(source[0], 4, @buffers);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceQueueBuffers 4: ', error);
        end;
      '5': begin
          alSourceQueueBuffers(source[0], 1, @buffers[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceQueueBuffers 1: ', error);
        end;
      '6': begin
          alSourceQueueBuffers(source[0], 1, @buffers[1]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceQueueBuffers 1: ', error);
        end;
      '7': begin
          alSourceQueueBuffers(source[0], 1, @buffers[2]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceQueueBuffers 1: ', error);
        end;
      '8': begin
          alSourceQueueBuffers(source[0], 1, @buffers[3]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceQueueBuffers 1: ', error);
        end;
      '9': begin
 // Queue buffer 0
          alSourceQueueBuffers(source[0], 1, @buffers[4]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceQueueBuffers 1(buffer 0): ', error);
        end;
      'A': begin
 // Unqueue first Buffer
          SetLength(buffersremoved, 1);
          alSourceUnqueueBuffers(source[0], 1, @buffersremoved);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceUnqueueBuffers 1: ', error);

          if (buffersremoved[0] = buffers[0]) then
            buffersremoved[0] := 1
          else if (buffersremoved[0] = buffers[1]) then
            buffersremoved[0] := 2
          else if (buffersremoved[0] = buffers[2]) then
            buffersremoved[0] := 3
          else if (buffersremoved[0] = buffers[3]) then
            buffersremoved[0] := 4
          else
            buffersremoved[0] := 0;
          write('Removed Buffer ', buffersremoved[0], ' from queue' + #13);
//					dispose(@buffersremoved);
        end;
      'B': begin
 // Unqueue first 2 Buffers
          SetLength(buffersremoved, 2);
          alSourceUnqueueBuffers(source[0], 2, @buffersremoved);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceUnqueueBuffers 2: ', error);

          for i := 0 to 1 do
          begin
            if (buffersremoved[i] = buffers[0]) then
              buffersremoved[i] := 1
            else if (buffersremoved[i] = buffers[1]) then
              buffersremoved[i] := 2
            else if (buffersremoved[i] = buffers[2]) then
              buffersremoved[i] := 3
            else if (buffersremoved[i] = buffers[3]) then
              buffersremoved[i] := 4
            else
              buffersremoved[i] := 0;
          end;

          write('Removed Buffers ', buffersremoved[0], ' and ',
            buffersremoved[1], ' from queue' + #13);

//          delete buffersremoved;
        end;
      'C': begin
 // Unqueue first 3 Buffers
          SetLength(buffersremoved, 3);
          alSourceUnqueueBuffers(source[0], 3, @buffersremoved);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceUnqueueBuffers 3: ', error);

          for i := 0 to 2 do
          begin
            if (buffersremoved[i] = buffers[0]) then
              buffersremoved[i] := 1
            else if (buffersremoved[i] = buffers[1]) then
              buffersremoved[i] := 2
            else if (buffersremoved[i] = buffers[2]) then
              buffersremoved[i] := 3
            else if (buffersremoved[i] = buffers[3]) then
              buffersremoved[i] := 4
            else
              buffersremoved[i] := 0;
          end;

          write('Removed Buffers ', buffersremoved[0], buffersremoved[1],
            ' and ', buffersremoved[2], ' from queue' + #13);
//					delete buffersremoved;
        end;
      'D': begin
 // Unqueue first 4 Buffers
          SetLength(buffersremoved, 4);
          alSourceUnqueueBuffers(source[0], 4, @buffersremoved);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceUnqueueBuffers 1: ', error);

          for i := 0 to 3 do
          begin
            if (buffersremoved[i] = buffers[0]) then
              buffersremoved[i] := 1
            else if (buffersremoved[i] = buffers[1]) then
              buffersremoved[i] := 2
            else if (buffersremoved[i] = buffers[2]) then
              buffersremoved[i] := 3
            else if (buffersremoved[i] = buffers[3]) then
              buffersremoved[i] := 4
            else
              buffersremoved[i] := 0;
          end;

          write('Removed Buffers ', buffersremoved[0], buffersremoved[1],
            buffersremoved[2], ' and ', buffersremoved[3], ' from queue' + #13);

//					delete buffersremoved;
        end;
      'E': begin
 // Unqueue first 5 Buffers
          SetLength(buffersremoved, 5);
          alSourceUnqueueBuffers(source[0], 5, @buffersremoved);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceUnqueueBuffers 1: ', error);

          for i := 0 to 4 do
          begin
            if (buffersremoved[i] = buffers[0]) then
              buffersremoved[i] := 1
            else if (buffersremoved[i] = buffers[1]) then
              buffersremoved[i] := 2
            else if (buffersremoved[i] = buffers[2]) then
              buffersremoved[i] := 3
            else if (buffersremoved[i] = buffers[3]) then
              buffersremoved[i] := 4
            else
              buffersremoved[i] := 0;
          end;

          write('Removed Buffers ', buffersremoved[0], buffersremoved[1],
            buffersremoved[2], buffersremoved[3], ' and ', buffersremoved[0],
            ' from queue' + #13);
//					delete buffersremoved;
        end;
      'F': begin
          alSourcei(source[0], AL_BUFFER, 0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSource AL_BUFFER NULL: ', error);
        end;
      '0': begin
 // Retrieve number of buffers in queue
          alGetSourcei(source[0], AL_BUFFERS_QUEUED, @BuffersInQueue);
 // Retrieve number of processed buffers
          alGetSourcei(source[0], AL_BUFFERS_PROCESSED, @BuffersProcessed);
 // Retrieve current buffer
          alGetSourcei(source[0], AL_BUFFER, @Buffer);
          if (Buffer = buffers[0]) then
            Buffer := 1
          else if (Buffer = buffers[1]) then
            Buffer := 2
          else if (Buffer = buffers[2]) then
            Buffer := 3
          else if (Buffer = buffers[3]) then
            Buffer := 4
          else
            Buffer := 0;

          Write('Current Buffer is ', Buffer, BuffersInQueue,
            ' Buffers in queue, ', BuffersProcessed, ' Processed' + #13);
        end;
    end;
  until (Key = 'Q');
  alSourceStopv(1, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alSourceStopv 1 : ' + IntToStr(error));

  alDeleteSources(1, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alDeleteSources 1 : ' + IntToStr(error));
end;

procedure BufferTest;
const
  source0Pos: array[0..2] of TALfloat = (2.0, 0.0, -2.0);
 // Front and right of the listener
  source0Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
var
  source: array[0..0] of TALuint;
  Key: TALubyte;
begin
  alGetError;

  alGenSources(1, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alGenSources Error: ' + IntToStr(error));
    exit;
  end;

  alSourcef(source[0], AL_PITCH, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_PITCH Error: ' + IntToStr(error));
    exit;
  end;
  alSourcef(source[0], AL_GAIN, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_GAIN Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_POSITION, @source0Pos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_POSITION Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_VELOCITY, @source0Vel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_VELOCITY Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[0], AL_LOOPING, AL_FALSE);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 0 AL_LOOPING Error: ' + IntToStr(error));
    exit;
  end;

  Writeln;
  Writeln('Buffer Test');
  Writeln('Press "1" to play buffer 0 on source 0');
  Writeln('Press "2" to play buffer 1 on source 0');
  Writeln('Press "3" to stop source 0');
  Writeln('Press "q" to quit');

  repeat
    Key := ReadKey;
    Key := UpCase(Key);
    case Key of
      '1': begin
 // Stop source
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop 0 : ', error);
 // Attach buffer 0 to source
          alSourcei(source[0], AL_BUFFER, g_Buffers[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei AL_BUFFER 0 : ', error);
 // Play
          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay 0 : ', error);
        end;
      '2': begin
 // Stop source
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop 0 : ', error);
 // Attach buffer 0 to source
          alSourcei(source[0], AL_BUFFER, g_Buffers[1]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei AL_BUFFER 1 : ', error);
 // Play
          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay 0 : ', error);
        end;
      '3': begin
 // Stop source
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop 0 : ', error);
        end;
    end;
  until (Key = 'Q');
  alSourceStopv(1, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alSourceStopv 1 : ' + IntToStr(error));

  alDeleteSources(1, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alDeleteSources 1 : ' + IntToStr(error));
end;

(*
 FreqTest
 Test AL_PITCH functionality
*)

procedure FreqTest;
const
  source0Pos: array[0..2] of TALfloat = (2.0, 0.0, -2.0);
 // Front and right of the listener
  source0Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
var
  error: TALint;
  source: array[0..0] of TALuint;
  Key: TALubyte;
begin
  alGetError;

  alGenSources(1, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alGenSources Error: ' + IntToStr(error));
    exit;
  end;

  alSourcef(source[0], AL_PITCH, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_PITCH Error: ' + IntToStr(error));
    exit;
  end;
  alSourcef(source[0], AL_GAIN, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_GAIN Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_POSITION, @source0Pos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_POSITION Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_VELOCITY, @source0Vel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_VELOCITY Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[0], AL_BUFFER, g_Buffers[1]);
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 0 AL_BUFFER Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[0], AL_LOOPING, AL_TRUE);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 0 AL_LOOPING Error: ' + IntToStr(error));
    exit;
  end;

  Writeln;
  Writeln('Frequency Test');
  Writeln('Press "1" to play source 0 (looping)');
  Writeln('Press "2" to Double Frequency');
  Writeln('Press "3" to Halve Frequency');
  Writeln('Press "q" to quit');
  repeat
    Key := ReadKey;
    Key := UpCase(Key);
    case Key of
      '1': begin
          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 0 : ', error);
        end;
      '2': begin
          alSourcef(source[0], AL_PITCH, 2.0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcef source 0 AL_PITCH 2.0 : ', error);
        end;
      '3': begin
          alSourcef(source[0], AL_PITCH, 0.5);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcef source 0 AL PITCH 0.5: ', error);
        end;
    end;
  until (Key = 'Q');
end;

(*
 Stereo Test
 Plays a stereo buffer
*)

procedure StereoTest;
const
  source0Pos: array[0..2] of TALfloat = (2.0, 0.0, -2.0);
 // Front and right of the listener
  source0Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
var
  source: array[0..0] of TALuint;
  Key: TALubyte;
  buffers: array[0..1] of TALuint;
  Buffer: TALuint;
  BuffersInQueue, BuffersProcessed: TALint;
  bLoop: TALint;
begin
  bLoop := AL_TRUE;
  alGenSources(1, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alGenSources 1 Error: ' + IntToStr(error));
    exit;
  end;

  buffers[0] := g_Buffers[6];
	buffers[1] := g_Buffers[6];

  Writeln('Stereo Test');
  Writeln('Press "1" to play a stereo buffer on source 0 (looping)');
  Writeln('Press "2" to play a mono buffer on source 0 (looping)');
  Writeln('Press "3" to stop source 0');
  Writeln('Press "4" to queue 2 stereo buffers on source 0 and start playing');
  Writeln('Press "5" to unqueue the 2 stereo buffers on source 0');
  Writeln('Press "6" to toggle looping on / off');
  Writeln('Press "0" to display stats');
  Writeln('Press "q" to quit');
  Write('Looping is on' + #13);

  repeat
    Key := ReadKey;
    Key := UpCase(Key);
    case Key of
      '1': begin
    // Stop source
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop source 0 : ', error);

  // Attach new buffer
          alSourcei(source[0], AL_BUFFER, g_Buffers[6]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei 0 AL_BUFFER buffer 6 (stereo) : \n', error);

    // Set volume
          alSourcef(source[0], AL_GAIN, 0.5);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcef 0 AL_GAIN : \n', error);

  // Set looping
          alSourcei(source[0], AL_LOOPING, bLoop);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei 0 AL_LOOPING true: \n', error);

  // Play source
          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 0 : ', error);

        end;
      '2': begin
  // Stop source
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop source 0 : ', error);

  // Attach new buffer
          alSourcei(source[0], AL_BUFFER, g_Buffers[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei 0 AL_BUFFER buffer 0 (mono) : ', error);

  // Set 3D position
          alSourcefv(source[0], AL_POSITION, @source0Pos);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcefv 0 AL_POSITION : ', error);

 // Set 3D velocity
          alSourcefv(source[0], AL_VELOCITY, @source0Vel);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcefv 0 AL_VELOCITY : ', error);

 // Set volume to full
          alSourcef(source[0], AL_GAIN, 1.0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcef 0 AL_GAIN : ', error);

 // Set Looping
          alSourcei(source[0], AL_LOOPING, bLoop);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei 0 AL_LOOPING : ', error);

 // Play source
          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 0 : ', error);
        end;
      '3': begin
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop source 0 : ', error);
        end;
      '4': begin
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop Source 0 : ', error);

 // Attach NULL buffer to source to clear everything
          alSourcei(source[0], AL_BUFFER, 0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei AL_BUFFER (NULL) : ', error);

          alSourceQueueBuffers(source[0], 2, @buffers);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceQueueBuffers 2 (stereo) : ', error);

 // Set Looping
          alSourcei(source[0], AL_LOOPING, bLoop);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei 0 AL_LOOPING : \n', error);

          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay Source 0 : ', error);
        end;
      '5': begin
          alSourceUnqueueBuffers(source[0], 2, @buffers);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceUnqueueBuffers 2 (stereo) : ', error);
        end;
      '6': begin
          if (bLoop = AL_TRUE) then
          begin
						Write('Looping is off' + #13);
            bLoop := AL_FALSE;
          end
          else
          begin
						Write('Looping is on ' + #13);
            bLoop := AL_TRUE;
          end;
          alSourcei(source[0], AL_LOOPING, bLoop);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcei 0 AL_LOOPING : ', error);
        end;
      '0': begin
 // Retrieve number of buffers in queue
          alGetSourcei(source[0], AL_BUFFERS_QUEUED, @BuffersInQueue);
 // Retrieve number of processed buffers
          alGetSourcei(source[0], AL_BUFFERS_PROCESSED, @BuffersProcessed);
 // Retrieve current buffer
          alGetSourcei(source[0], AL_BUFFER, @Buffer);
          if (Buffer = buffers[0]) then
            Buffer := 6
          else if (Buffer = buffers[1]) then
            Buffer := 6
          else
            Buffer := 0;

					Write('Current Buffer is ', Buffer, ', ', BuffersInQueue,
            ' Buffers in queue, ', BuffersProcessed, ' Processed' + #13);

        end;
    end;
  until (Key = 'Q');
 // Release resources
  alSourceStop(source[0]);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alSourceStop : ', error);

  alDeleteSources(1, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alDeleteSources 2 : ', error);
end;
(*
 GainTest
 Play 2 Sources - control gain of each, and control Listener Gain
*)

procedure GainTest;
const
  source0Pos: array[0..2] of TALfloat = (2.0, 0.0, -2.0);
 // Front and right of the listener
  source0Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
  source1Pos: array[0..2] of TALfloat = (-2.0, 0.0, -2.0);
  source1Vel: array[0..2] of TALfloat = (0.0, 0.0, 0.0);
var
  source: array[0..1] of TALuint;
  Key: TALubyte;
begin
  Writeln;
  alGenSources(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alGenSources Error: ' + IntToStr(error));
    exit;
  end;

  alSourcef(source[0], AL_PITCH, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_PITCH Error: ' + IntToStr(error));
    exit;
  end;
  alSourcef(source[0], AL_GAIN, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 0 AL_GAIN Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_POSITION, @source0Pos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_POSITION Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[0], AL_VELOCITY, @source0Vel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 0 AL_VELOCITY Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[0], AL_BUFFER, g_Buffers[0]);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 0 AL_BUFFER Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[0], AL_LOOPING, AL_TRUE);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 0 AL_LOOPING Error: ' + IntToStr(error));
    exit;
  end;

  alSourcef(source[1], AL_PITCH, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 1 AL_PITCH Error: ' + IntToStr(error));
    exit;
  end;
  alSourcef(source[1], AL_GAIN, 1.0);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcef 1 AL_GAIN Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[1], AL_POSITION, @source0Pos);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 1 AL_POSITION Error: ' + IntToStr(error));
    exit;
  end;
  alSourcefv(source[1], AL_VELOCITY, @source0Vel);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcefv 1 AL_VELOCITY Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[1], AL_BUFFER, g_Buffers[1]);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 1 AL_BUFFER Error: ' + IntToStr(error));
    exit;
  end;
  alSourcei(source[1], AL_LOOPING, AL_TRUE);
  error := alGetError;
  if error <> AL_NO_ERROR then
  begin
    Writeln('alSourcei 1 AL_LOOPING Error: ' + IntToStr(error));
    exit;
  end;
  Writeln;
  Writeln('Gain Test');
  Writeln('Press "1" to play source 0 (looping)');
  Writeln('Press "2" to play source 1 (looping)');
  Writeln('Press "3" to stop source 0');
  Writeln('Press "4" to stop source 1');
  Writeln('Press "5" to set source 0 gain to 1.0');
  Writeln('Press "6" to set source 0 gain to 0.5');
  Writeln('Press "7" to set source 0 gain to 0.25');
  Writeln('Press "8" to set source 0 gain to 0');
  Writeln('Press "a" to set Listener Gain to 1.0');
  Writeln('Press "b" to set Listener Gain to 0.5');
  Writeln('Press "c" to set Listener Gain to 0.25');
  Writeln('Press "d" to set Listener Gain to 0.0');
  Writeln('Press "q" to quit');
  repeat
    Key := ReadKey;
    Key := UpCase(Key);
    case Key of
      '1': begin
          alSourcePlay(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 0 : ', error);
        end;
      '2': begin
          alSourcePlay(source[1]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcePlay source 1 : ', error);
        end;
      '3': begin
          alSourceStop(source[0]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop source 0 : \n', error);
        end;
      '4': begin
          alSourceStop(source[1]);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourceStop source 1 : \n', error);
        end;
      '5': begin
          alSourcef(source[0], AL_GAIN, 1.0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcef 0 AL_GAIN 1.0 : \n', error);
        end;
      '6': begin
          alSourcef(source[0], AL_GAIN, 0.5);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcef 0 AL_GAIN 0.5 : \n', error);
        end;
      '7': begin
          alSourcef(source[0], AL_GAIN, 0.25);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcef 0 AL_GAIN 0.25 : \n', error);
        end;
      '8': begin
          alSourcef(source[0], AL_GAIN, 0.0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alSourcef 0 AL_GAIN 0.0 : \n', error);
        end;
      'A': begin
          alListenerf(AL_GAIN, 1.0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alListenerf AL_GAIN 1.0 : \n', error);
        end;
      'B': begin
          alListenerf(AL_GAIN, 0.5);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alListenerf AL_GAIN 0.5 : \n', error);
        end;
      'C': begin
          alListenerf(AL_GAIN, 0.25);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alListenerf AL_GAIN 0.25 : \n', error);
        end;
      'D': begin
          alListenerf(AL_GAIN, 0.0);
          error := alGetError;
          if error <> AL_NO_ERROR then
            Writeln('alListenerf AL_GAIN 0.0 : \n', error);
        end;
    end;
  until (Key = 'Q');
  alSourceStopv(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alSourceStop : ', error);

  alDeleteSources(2, @source);
  error := alGetError;
  if error <> AL_NO_ERROR then
    Writeln('alDeleteSources 2 : ', error);
end;

(*
 Streaming Test
 Stream a long stereo wave file from harddisc - using AL Queuing
*)

procedure StreamingTest;
var
  Stream: TStream;
  wave: TWaveStruct;
  data: array[0..BSIZE - 1] of TALbyte;
  Buffers: array[0..NUMBUFFERS - 1] of TALuint;
  BufferID: TALuint;
  Sources: array[0..0] of TALuint;
  DataSize: TALuint;
  DataToRead: TALuint;
  processed: TALint;
  bFinished: TALint;
  Format: TALuint;
  loop: TALuint;

	buffersreturned: TALuint;
  bFinishedPlaying: TALint;
  buffersinqueue: TALuint;

begin
  Writeln;
  Writeln('Streaming Test');
  Stream := TFileStream.Create('data\stereo.wav', fmOpenRead);
  try
    if Stream <> nil then
    begin
      Stream.Read(wave, sizeof(TWaveStruct));
      DataSize := wave.dataSize;
      if (wave.Channels = 1) then
        Format := AL_FORMAT_MONO16
      else
        Format := AL_FORMAT_STEREO16;
      alGenBuffers(NUMBUFFERS, @Buffers);
      error := alGetError;
      if error <> AL_NO_ERROR then
      begin
        Writeln('alGenBuffers : ', error);
        exit
      end;
      for loop := 0 to NUMBUFFERS - 1 do
      begin
  //fread(data, 1, BSIZE, fp);
        Stream.Read(data, BSIZE);
        DataSize := DataSize - BSIZE;
        alBufferData(Buffers[loop], Format, @data, BSIZE, wave.SamplesPerSec);
        error := alGetError;
        if error <> AL_NO_ERROR then
          Writeln('alBufferData : ', error);
      end;
      alGenSources(1, @Sources);
      error := alGetError;
      if error <> AL_NO_ERROR then
      begin
        Writeln('alGenSources 1 : ', error);
        exit
      end;

      alSourcef(Sources[0], AL_GAIN, 0.5);
      error := alGetError;
      if error <> AL_NO_ERROR then
        Writeln('alSourcef AL_GAIN: ', error);

 // Queue the buffers on the source
      alSourceQueueBuffers(Sources[0], NUMBUFFERS, @Buffers);
      error := alGetError;
      if error <> AL_NO_ERROR then
        Writeln('alSourceQueueBuffers: ', error);

      alSourcei(Sources[0], AL_LOOPING, AL_FALSE);

 // Start playing source
      alSourcePlay(Sources[0]);
      error := alGetError;
      if error <> AL_NO_ERROR then
        Writeln('alSourcePlay source 0: ', error);

      Writeln('Please wait ...');

			buffersreturned := 0;
      bFinishedPlaying := AL_FALSE;
      buffersinqueue := NUMBUFFERS;

      while {not KeyPressed and}(bFinishedPlaying = AL_FALSE) do
      begin
 // Get status
        alGetSourcei(Sources[0], AL_BUFFERS_PROCESSED, @processed);
        if (processed > 0) then
        begin
					buffersreturned := buffersreturned + processed;
          Write('Buffers Completed is ', buffersreturned, #13);
 // Pseudo code for Streaming with Open AL
 // while (processed)
 //		Unqueue a buffer
 //		Load audio data into buffer (returned by UnQueueBuffers)
 //		if successful
 //			Queue buffer
 //			processed--
 //		else
 //			buffersinqueue--
 //			if buffersinqueue == 0
 //				finished playing !
          while (processed <> 0) do
          begin
            alSourceUnqueueBuffers(Sources[0], 1, @BufferID);
            error := alGetError;
            if error <> AL_NO_ERROR then
              Writeln('alSourceUnqueueBuffers 1 : ', error);
//						if (bFinished = AL_FALSE) then
						if (bFinished <> AL_FALSE) then
            begin
 //DataToRead = (DataSize > BSIZE) ? BSIZE : DataSize;
              if DataSize > BSIZE then
                DataToRead := BSIZE
              else
                DataToRead := DataSize;
              if (DataToRead = DataSize) then
                bFinished := AL_FALSE;
 //fread(data, 1, DataToRead, fp);
              Stream.Read(data, DataToRead);
              DataSize := DataSize - DataToRead;
              if (bFinished = AL_FALSE) then
//						memset(data + DataToRead, 0, BSIZE - DataToRead)
                FillChar(data[DataToRead], BSIZE - DataToRead, 0);
              ;
              alBufferData(BufferID, Format, @data, DataToRead,
                wave.SamplesPerSec);
              error := alGetError;
              if error <> AL_NO_ERROR then
                Writeln('alBufferData : ', error);

 // Queue buffer
              alSourceQueueBuffers(Sources[0], 1, @BufferID);
              error := alGetError;
              if error <> AL_NO_ERROR then
                Writeln('alSourceQueueBuffers 1 : ', error);
              processed := processed - 1;
            end else
            begin
              buffersinqueue := buffersinqueue - 1;
              processed := processed - 1;
              if (buffersinqueue = 0) then
              begin
                bFinishedPlaying := AL_TRUE;
                break;
              end;
            end;
          end;
        end;
      end;

      alSourceStopv(1, @sources);
      error := alGetError;
      if error <> AL_NO_ERROR then
        Writeln('alSourceStopv 1 : ' + IntToStr(error));

      alDeleteSources(1, @sources);
      error := alGetError;
      if error <> AL_NO_ERROR then
        Writeln('alDeleteSources 1 : ' + IntToStr(error));

      alDeleteBuffers(NUMBUFFERS, @Buffers);
      error := alGetError;
      if error <> AL_NO_ERROR then
        Writeln('alDeleteBuffers : ', error);
    end;
  finally
    Stream.Free;
  end;
end;

(*
 Main application - Initializes Open AL, Sets up Listener, Generates Buffers, and loads in audio data.
 Displays Test Menu and calls appropriate Test functions.  On exit, buffers are destroyed and Open AL
 is shutdown.

 Each Test function is responsible for creating and destroying whatever Sources they require. All test
 applications use the same set of Buffers (created in this function).
*)
begin
  Writeln('OpenAL Test Application');
  Writeln;
  try
    if InitAL then
    begin
 //Listenet Attributes
      if not SetListernerAttributes then
        Exit;
 //Generate Buffers
      alGenBuffers(NUM_BUFFERS, @g_Buffers);
      error := alGetError;
      if error <> AL_NO_ERROR then
      begin
        Writeln('alGenBuffers error');
        Exit;
      end;
 // Load in samples to be used by Test functions
      LoadSample('data\footsteps.wav', 0);
      LoadSample('data\ding.wav', 1);
      LoadSample('data\wave1.wav', 2);
      LoadSample('data\wave2.wav', 3);
      LoadSample('data\wave3.wav', 4);
      LoadSample('data\wave4.wav', 5);
      LoadSample('data\stereo.wav', 6);

 // Check for EAX 2.0 support
      g_bEAX := alIsExtensionPresent('EAX2.0');

      if (g_bEAX) then
        Writeln('EAX 2.0 Extension available');
//      szFnName := 'EAXSet';

//	eaxSet := (EAXSet)alGetProcAddress(szFnName);
//	if (eaxSet == NULL)
//		g_bEAX = false;
//      szFnName := 'EAXGet';
//	eaxGet = (EAXGet)alGetProcAddress(szFnName);
//	if (eaxGet == NULL)
//		g_bEAX = false;

      Writeln;
      repeat
        Writeln;
        Writeln('Select a test from the following options');
        writeln('1 Position Test');
        writeln('2 Looping Test');
        writeln('3 EAX 2.0 Test');
        writeln('4 Queue Test');
        writeln('5 Buffer Test');
        writeln('6 Frequency Test');
        writeln('7 Stereo Test');
        writeln('8 Gain Test');
        writeln('9 Streaming Test');
        writeln('Q to quit');

        ch := ReadKey;
        ch := UpCase(ch);

        case (ch) of
          '1': PositionTest();
          '2': LoopingTest();
          '3': begin
              if (g_bEAX) then
                EAXTest()
              else
                Writeln('EAX Extensions NOT available.  Please select another test');
            end;
          '4': QueueTest();
          '5': BufferTest();
          '6': FreqTest();
          '7': StereoTest();
          '8': GainTest();
          '9': StreamingTest();
        end;
      until (ch = 'Q');
      alDeleteBuffers(NUM_BUFFERS, @g_Buffers);
      error := alGetError;
      if (error <> AL_NO_ERROR) then
      begin
        Writeln('alDeleteBuffers Error: ', error);
        Exit;
      end;
      Writeln('Press Any key to exit...');
    end;
  finally
    Cleanup;
    Writeln('Bye!');
  end;
end.

